const path = require('path')

function addWorkspaceToArgumentsIfExists (args, workspace) {
  if (workspace) {
    args.unshift(workspace)
  }
  return args
}

// normalize file/line numbers into command line args for specific editors
module.exports = function getArgumentsForLineNumber (
  editor,
  fileName,
  lineNumber,
  workspace
) {
  const editorBasename = path.basename(editor).replace(/\.(exe|cmd|bat)$/i, '')
  switch (editorBasename) {
  case 'atom':
  case 'Atom':
  case 'Atom Beta':
  case 'subl':
  case 'sublime':
  case 'sublime_text':
  case 'wstorm':
  case 'charm':
    return [fileName + ':' + lineNumber]
  case 'notepad++':
    return ['-n' + lineNumber, fileName]
  case 'vim':
  case 'mvim':
  case 'joe':
  case 'emacs':
  case 'emacsclient':
    return ['+' + lineNumber, fileName]
  case 'rmate':
  case 'mate':
  case 'mine':
    return ['--line', lineNumber, fileName]
  case 'code':
  case 'Code':
    return addWorkspaceToArgumentsIfExists(
      ['-g', fileName + ':' + lineNumber],
      workspace
    )
  case 'appcode':
  case 'clion':
  case 'clion64':
  case 'idea':
  case 'idea64':
  case 'phpstorm':
  case 'phpstorm64':
  case 'pycharm':
  case 'pycharm64':
  case 'rubymine':
  case 'rubymine64':
  case 'webstorm':
  case 'webstorm64':
    return addWorkspaceToArgumentsIfExists(
      ['--line', lineNumber, fileName],
      workspace
    )
  }

  // For all others, drop the lineNumber until we have
  // a mapping above, since providing the lineNumber incorrectly
  // can result in errors or confusing behavior.
  return [fileName]
}
